/*  
    Free and open source, CC BY-SA 4.0
    
    https://espressif.com/sites/default/files/documentation/esp-now_user_guide_en.pdf
    https://docs.espressif.com/projects/esp-idf/en/stable/api-reference/wifi/esp_now.html
   
    http://www.arduino.cc/en/Tutorial/Smoothing
*/

//ESP-Now WiFi power saving protocol
#include <esp_now.h>
#include <WiFi.h>

// Channel used for connection
#define CHANNEL 1

uint8_t data;

uint16_t period = 2000;
unsigned long time_now = 0;

// Azimuth and elevation pwm (pulse width modulation) pins on each h-bridge must be connected to four pwm pins on microprocessor.
// For ESP32 the GPIO pwm pins are: 0,1,2,3,4,6,7,8,9,10,11,12,14,16,17,18,19,21,22,23,25,26,27,32,33,34,35,36,39

// motor azimuth adjustment
uint8_t AzimuthClockwiseEnable = 26;
uint8_t AzimuthCounterClockwiseEnable = 27;

uint8_t AzimuthClockwisePWM = 32;
uint8_t AzimuthCounterClockwisePWM = 33;

// motor elevation adjustment
uint8_t ElevationClockwiseEnable = 34;
uint8_t ElevationCounterClockwiseEnable = 35;

uint8_t ElevationClockwisePWM = 36;
uint8_t ElevationCounterClockwisePWM = 37;

// Setting PWM properties
uint16_t freq = 30000;
uint8_t resolution = 8;
uint8_t DutyCycleAzimuthCW = 200;
uint8_t DutyCycleAzimuthCCW = 200;
uint8_t DutyCycleElevationCW = 200;
uint8_t DutyCycleElevationCCW = 200;

void setup() {
  Serial.begin(115200);

  // Call the function that initializes the station mode
  modeStation();

  // Call function that initializes ESPNow
  InitESPNow();

  // Registers the callback that will inform us that we receive data. The function to be executed is onDataRecv and is declared below
  esp_now_register_recv_cb(onDataRecv);

  // Registers the callback that will inform us
  // about the status of the submission.
  // The function to be executed
  // is onDataSent and is declared below
  esp_now_register_send_cb(onDataSent);

  //set all motor control pins to outputs
  pinMode(AzimuthClockwiseEnable, OUTPUT);
  pinMode(AzimuthCounterClockwiseEnable, OUTPUT);

  pinMode(AzimuthClockwisePWM, OUTPUT);
  pinMode(AzimuthCounterClockwisePWM, OUTPUT);

  pinMode(ElevationClockwiseEnable, OUTPUT);
  pinMode(ElevationCounterClockwiseEnable, OUTPUT);

  pinMode(ElevationClockwisePWM, OUTPUT);
  pinMode(ElevationCounterClockwisePWM, OUTPUT);

  // configure LED PWM functionalitites
  ledcSetup(AzimuthClockwisePWM, freq, resolution);
  ledcSetup(AzimuthCounterClockwisePWM, freq, resolution);

  ledcSetup(ElevationClockwisePWM, freq, resolution);
  ledcSetup(ElevationCounterClockwisePWM, freq, resolution);

  // attach the channel to the GPIO to be controlled
  ledcAttachPin(AzimuthClockwiseEnable, AzimuthClockwisePWM);
  ledcAttachPin(AzimuthCounterClockwiseEnable, AzimuthCounterClockwisePWM);

  ledcAttachPin(ElevationClockwiseEnable, ElevationClockwisePWM);
  ledcAttachPin(ElevationCounterClockwiseEnable, ElevationCounterClockwisePWM);
}

// We send the data as soon as we get from the other esp by the callback
void loop()
{
  if (millis() > time_now + period) 
  {
    time_now = millis();
  }

  if (data = 0) // if average light sensor values on left side are greater than right side, azimuth motor rotates clockwise
  {
    digitalWrite(AzimuthClockwiseEnable, HIGH); ledcWrite(AzimuthClockwisePWM, DutyCycleAzimuthCW); // set speed out of possible range 0~255
    digitalWrite(AzimuthCounterClockwiseEnable, HIGH); digitalWrite (AzimuthCounterClockwisePWM, LOW);
    Serial.println(data);
    Serial.println("AZIMUTH MOTOR MOVES CLOCKWISE");
    Serial.println("   ");
  }
  else if (data = 1)// if average light sensor values on right side are greater than on left side, azimuth motor rotates counterclockwise
  {
    digitalWrite(AzimuthCounterClockwiseEnable, HIGH); ledcWrite(AzimuthCounterClockwisePWM, DutyCycleAzimuthCCW);
    digitalWrite(AzimuthClockwiseEnable, HIGH); digitalWrite(AzimuthClockwisePWM, LOW);
    Serial.println(data);
    Serial.println("AZIMUTH MOTOR MOVES COUNTERCLOCKWISE");
    Serial.println(" ");
  }
  else if (data = 2) // if difference is smaller than tolerance, stop azimuth motor
  {
    digitalWrite(AzimuthClockwiseEnable, LOW); digitalWrite(AzimuthCounterClockwiseEnable, LOW);
    Serial.println("AZIMUTH MOTOR STOPS");
    Serial.println(" ");
  }

  if (data = 3) // if average light sensor value on top side is greater than on bottom side then elevation motor rotates clockwise
  {
    digitalWrite(ElevationClockwiseEnable, HIGH); ledcWrite(ElevationClockwisePWM, DutyCycleElevationCW);
    digitalWrite(ElevationCounterClockwiseEnable, HIGH); digitalWrite(ElevationCounterClockwisePWM, LOW);
    Serial.println(data);
    Serial.println("ELEVATION MOTOR MOVES CLOCKWISE");
    Serial.println(" ");
  }
  else if (data = 4) // if average light sensor value on bottom side is greater than on top side then elevation motor rotates counterclockwise
  {
    digitalWrite(ElevationCounterClockwiseEnable, HIGH); ledcWrite(ElevationCounterClockwisePWM, DutyCycleElevationCCW);
    digitalWrite(ElevationClockwiseEnable, HIGH); digitalWrite(ElevationClockwisePWM, LOW);
    Serial.println(data);
    Serial.println("ELEVATION MOTOR MOVES COUNTERCLOCKWISE");
    Serial.println(" ");
  }
  else if (data = 5) // if difference is smaller than tolerance, stop elevation motor
  {
    digitalWrite(ElevationClockwiseEnable, LOW); digitalWrite(ElevationCounterClockwiseEnable, LOW);
    Serial.println(data);
    Serial.println("ELEVATION MOTOR STOPS");
    Serial.println(" ");
  }
  if (data = 6)
  {
    digitalWrite(ElevationClockwiseEnable, LOW); digitalWrite(ElevationCounterClockwiseEnable, LOW);
    Serial.println(data);
    Serial.println("ELEVATION MOTOR STOPS");
    Serial.println(" ");
  }
}


// Function that serves as a callback to warn us we received data
void onDataRecv(const uint8_t *mac_addr, const uint8_t *data, int len) {
  // Place the received data at the output of the pin
  Serial.println(*data);
}

// Function that serves as a callback to warn us about the sending situation we made
void onDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Success" : "Fail");
}

// Function to initialize the station mode
void modeStation() {
  // We put the ESP in station mode
  WiFi.mode(WIFI_STA);
  // We show on Serial Monitor the Mac Address
  // of this ESP when in station mode
  Serial.print("Mac Address in Station: ");
  Serial.println(WiFi.macAddress());
}

// ESPNow startup function
void InitESPNow() {
  // If the initialization was successful
  if (esp_now_init() == ESP_OK) {
    Serial.println("ESPNow Init Success");
  }
  // If initialization failed
  else {
    Serial.println("ESPNow Init Failed");
    ESP.restart();
  }
}
